package com.scalapay.converters;

import com.google.common.base.Strings;
import com.scalapay.data.*;
import de.hybris.platform.acceleratorservices.urlresolver.SiteBaseUrlResolutionService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.converters.Populator;
import de.hybris.platform.core.model.c2l.CurrencyModel;
import de.hybris.platform.core.model.order.AbstractOrderEntryModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.price.DiscountModel;
import de.hybris.platform.core.model.user.AddressModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.core.model.user.UserModel;
import de.hybris.platform.servicelayer.config.ConfigurationService;
import de.hybris.platform.servicelayer.dto.converter.ConversionException;
import de.hybris.platform.site.BaseSiteService;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

public class CartToTokenRequestPopulator implements Populator<CartModel, OrderTokenRequest> {

    private static final String SCALAPAY_REDIRECT_CONFIRM_URL = "scalapay.redirect.confirm.url";
    private static final String SCALAPAY_REDIRECT_CANCEL_URL = "scalapay.redirect.cancel.url";

    @Resource
    private ConfigurationService configurationService;

    @Resource
    private SiteBaseUrlResolutionService siteBaseUrlResolutionService;

    @Resource
    private BaseSiteService baseSiteService;

    @Override
    public void populate(CartModel cartModel, OrderTokenRequest request) throws ConversionException {
        String email = "";
        UserModel user = cartModel.getUser();
        if (user instanceof CustomerModel) {
            email = ((CustomerModel) user).getContactEmail();
            if (Strings.isNullOrEmpty(email)) {
                email = ((CustomerModel) user).getCustomerID();
            }

        }
        request.setTotalAmount(createAmount(cartModel.getCurrency(), cartModel.getTotalPrice()));
        request.setConsumer(createConsumer(cartModel.getDeliveryAddress(), email));
        request.setBilling(createAddress(cartModel.getPaymentAddress() != null ? cartModel.getPaymentAddress() : cartModel.getDeliveryAddress())); // todo revisit this
        request.setShipping(createAddress(cartModel.getDeliveryAddress()));
        request.setItems(createItems(cartModel.getEntries(), cartModel.getCurrency()));
        List<Discount> discount = createDiscount(cartModel);
        if (discount.size() > 0) {
            request.setDiscounts(discount);
        }
        request.setMerchant(createMerchant());
        request.setMerchantReference(createMerchantReference(cartModel));
        request.setTaxAmount(createAmount(cartModel.getCurrency(), cartModel.getTotalTax()));
        request.setShippingAmount(createAmount(cartModel.getCurrency(), cartModel.getDeliveryCost()));
        request.setOrderExpiryMilliseconds(3600000);
    }

    protected String createMerchantReference(CartModel cartModel) {
        return cartModel.getCode();
    }

    protected Merchant createMerchant() {
        Merchant merchant = new Merchant();
        String fullConfirmUrl = buildSiteUrl(lookupConfig(SCALAPAY_REDIRECT_CONFIRM_URL));
        merchant.setRedirectConfirmUrl(fullConfirmUrl);
        String fullFailureUrl = buildSiteUrl(lookupConfig(SCALAPAY_REDIRECT_CANCEL_URL));
        merchant.setRedirectCancelUrl(fullFailureUrl);
        return merchant;
    }

    protected String lookupConfig(final String key) {
        return configurationService.getConfiguration().getString(key, null);
    }

    protected List<Discount> createDiscount(CartModel cartModel) {
        List<Discount> discounts = new ArrayList<>();
        for (DiscountModel cartDiscount : cartModel.getDiscounts()) {
            Discount discount = new Discount();
            discount.setDisplayName(cartDiscount.getName());
            CurrencyModel currency = cartDiscount.getCurrency();
            if (currency == null) {
                currency = cartModel.getCurrency();
            }
            Amount amount = createAmount(currency, cartDiscount.getValue());
            discount.setAmount(amount);
            discounts.add(discount);
        }
        return discounts;
    }

    protected Amount createAmount(CurrencyModel currency, Double totalPrice) {
        Amount amount = new Amount();
        amount.setCurrency(currency.getIsocode());
        amount.setAmount(String.valueOf(totalPrice));
        return amount;
    }

    protected List<Item> createItems(List<AbstractOrderEntryModel> entries, CurrencyModel currency) {
        ArrayList<Item> items = new ArrayList<>();
        for (AbstractOrderEntryModel entry : entries) {
            items.add(createItem(entry, currency));
        }
        return items;
    }

    protected Item createItem(AbstractOrderEntryModel entry, CurrencyModel currency) {
        Item item = new Item();
        item.setName(entry.getProduct().getName());
        item.setSku(entry.getProduct().getCode());
        item.setPrice(createAmount(currency, entry.getBasePrice()));
        item.setQuantity(Math.toIntExact(entry.getQuantity()));
        return item;

    }

    protected Address createAddress(AddressModel cartAddress) {
        Address address = new Address();
        address.setName(cartAddress.getFirstname() + " " + cartAddress.getLastname());
        address.setLine1(cartAddress.getLine1());
        address.setSuburb(cartAddress.getTown());
        address.setPostcode(cartAddress.getPostalcode());
        address.setCountryCode(cartAddress.getCountry().getIsocode());
        address.setPhoneNumber(Strings.emptyToNull(cartAddress.getPhone1()));
        return address;
    }

    protected Customer createConsumer(AddressModel deliveryAddress, String email) {
        Customer customer = new Customer();
        customer.setPhoneNumber(Strings.emptyToNull(deliveryAddress.getPhone1()));
        customer.setGivenNames(deliveryAddress.getFirstname());
        customer.setSurname(deliveryAddress.getLastname());
        customer.setEmail(Strings.isNullOrEmpty(deliveryAddress.getEmail()) ? email : deliveryAddress.getEmail());
        return customer;
    }

    protected String buildSiteUrl(final String url) {

        final BaseSiteModel currentBaseSite = baseSiteService.getCurrentBaseSite();
        return siteBaseUrlResolutionService.getWebsiteUrlForSite(currentBaseSite, true, url);

    }

    public void setConfigurationService(ConfigurationService configurationService) {
        this.configurationService = configurationService;
    }

    public void setSiteBaseUrlResolutionService(SiteBaseUrlResolutionService siteBaseUrlResolutionService) {
        this.siteBaseUrlResolutionService = siteBaseUrlResolutionService;
    }

    public void setBaseSiteService(BaseSiteService baseSiteService) {
        this.baseSiteService = baseSiteService;
    }



}
