package com.scalapay.facade;

import com.scalapay.data.ScalaPayStylingData;
import de.hybris.platform.commerceservices.service.data.CommerceCheckoutParameter;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;

import javax.annotation.Nullable;
import java.util.Map;

public interface ScalapayFacade {

    /**
     * Checks if ScalaPay is enabled for the current store.
     * @return True is enabled, false otherwise.
     */
    boolean isEnabled();

    /**
     * Checks if Scalapay is supported for the given amount.
     * The check is taking into consideration the minimum and maximum amount of the Scalapay configuration, and the
     * currently selected for the current cart. The check does not validate if Scalapay is actually enabled, this can
     * be achieved by calling isEnabled().
     * @param amount Tge amount to check.
     * @return True is supported, false otherwise
     */
    boolean validForSessionCurrencyAndAmount(double amount);

    /**
     * Same as validForSessionCurrencyAndAmount(), but using the current cart value for the amount.
     * @return True if valid, false otherwise.
     */
    boolean validForSessionCart();

    /**
     * Calls the Scalapay API to retrieve an order token, together with the corresponding checkout URL
     * @return A single entry map where the key is the orderToken, and the value the checkout url.
     */
    Map<String, String> getOrderToken();

    /**
     * Adds the checkout token into the session
     * @param token The checkout token
     */
    void setSessionCheckoutToken(String token);

    /**
     * Retrieves the checkout token form the session.
     * @return The checkout token
     */
    String getSessionCheckoutToken();

    /**
     * Creates a PaymentModeModel with a new ScalaPayPaymentInfoModel instance for  the given token
     * and customer.
     * @param orderToken The order token
     * @param customerModel The customer
     */
    void createAndStorePayment(String orderToken, CustomerModel customerModel);

    /**
     * Captures the payment against the ScalaPay API
     * @param parameter The CommerceCheckoutParameter
     * @param reference The reference String
     * @return A new PaymentTransactionEntryModel
     */
    PaymentTransactionEntryModel capture(CommerceCheckoutParameter parameter, @Nullable String reference);

    /**
     * Updates the merchant reference. This might be required of the order number is not known at the time
     * of the order payment capture.
     * @param order The order to use as new reference
     */
    void updateMerchantReference(OrderModel order);

    /**
     * Get the min amount supported by teh scalapay configuration.
     * @return The minimum supported amount.
     */
    Integer getMinimumSupportedAmount();

    /**
     * Get the max amount supported by teh scalapay configuration.
     * @return The maximum supported amount.
     */
    Integer getMaximumSupportedAmount();

    /**
     * Get the styling data is a map where the key defines the site type (PDP, cart page)
     * @return The styling data
     */
    Map<String, ScalaPayStylingData> getStylingData();
}
