package com.scalapay.facade.impl;

import com.google.common.base.Joiner;
import com.google.common.base.Predicate;
import com.scalapay.data.*;
import com.scalapay.facade.ScalapayFacade;
import com.scalapay.model.ScalaPayConfigurationModel;
import com.scalapay.service.ScalapayService;
import de.hybris.platform.commerceservices.service.data.CommerceCheckoutParameter;
import de.hybris.platform.core.enums.PaymentStatus;
import de.hybris.platform.core.model.c2l.C2LItemModel;
import de.hybris.platform.core.model.c2l.CurrencyModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.PaymentModeModel;
import de.hybris.platform.core.model.order.payment.ScalaPayPaymentInfoModel;
import de.hybris.platform.core.model.user.CustomerModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.order.PaymentModeService;
import de.hybris.platform.payment.dto.TransactionStatus;
import de.hybris.platform.payment.model.PaymentTransactionEntryModel;
import de.hybris.platform.payment.model.PaymentTransactionModel;
import de.hybris.platform.servicelayer.dto.converter.Converter;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.session.SessionService;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Required;

import javax.annotation.Nullable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;


public class DefaultScalapayFacade implements ScalapayFacade {

    private static final Logger LOG = Logger.getLogger(DefaultScalapayFacade.class.getName());
    public static final String SCALA_CHECKOUT_TOKEN = "SCALA_CHECKOUT_TOKEN";
    public static final String SCALA_PAY_CODE = "Scalapay";

    private ScalapayService scalapayService;

    private CartService cartService;

    private Converter<CartModel, OrderTokenRequest> orderTokenRequestConverter;

    private SessionService sessionService;

    private PaymentModeService paymentModeService;

    private ModelService modelService;

    private Converter<ScalaPayConfigurationModel, Map<String, ScalaPayStylingData>> scalaPayStylingConverter;

    @Override
    public boolean isEnabled() {
        CartModel sessionCart = cartService.getSessionCart();
        return scalapayService.isEnabled(sessionCart.getStore());
    }

    @Override
    public boolean validForSessionCurrencyAndAmount(double amount) {
        CartModel cart = cartService.getSessionCart();
        ScalaPayConfigurationModel scalaPayConfiguration = getScalaPayConfiguration();
        Integer minAmount = scalaPayConfiguration.getMinAmount();
        Integer maxAmount = scalaPayConfiguration.getMaxAmount();


        CurrencyModel cartCurrency = cart.getCurrency();
        if (cartCurrency == null) {
            LOG.warn("No currency provided for cart");
            return false;
        }
        boolean supported = scalaPayConfiguration.getCurrencies().stream().anyMatch((Predicate<CurrencyModel>) currencyModel -> currencyModel == cartCurrency);
        if (!supported) {
            List<String> supportedCurrencyIsoCodes = scalaPayConfiguration.getCurrencies().stream().map(C2LItemModel::getIsocode).collect(Collectors.toList());
            LOG.warn("ScalaPay not supporting currency <" + cartCurrency + ">, only <" + Joiner.on(", ").join(supportedCurrencyIsoCodes) + " >");
            return false;
        }

        return amount >= (minAmount) && amount <= maxAmount;

    }

    @Override
    public boolean validForSessionCart() {
        CartModel cart = cartService.getSessionCart();
        Double totalCartPrice = cart.getTotalPrice();
        return validForSessionCurrencyAndAmount(totalCartPrice);
    }

    @Override
    public Map<String, String> getOrderToken() {
        CartModel sessionCart = cartService.getSessionCart();
        OrderTokenRequest request = orderTokenRequestConverter.convert(sessionCart);
        OrderTokenResponse orderToken = scalapayService.getOrderToken(request, sessionCart.getStore());
        if (orderToken == null) {
            return null;
        }
        HashMap<String, String> result = new HashMap<>();
        result.put(orderToken.getToken(), orderToken.getCheckoutUrl());
        return result;
    }

    @Override
    public void setSessionCheckoutToken(String token) {
        sessionService.setAttribute(SCALA_CHECKOUT_TOKEN, token);
    }

    @Override
    public String getSessionCheckoutToken() {
        return sessionService.getAttribute(SCALA_CHECKOUT_TOKEN);
    }

    @Override
    public void createAndStorePayment(String orderToken, CustomerModel customerModel) {
        CartModel cart = cartService.getSessionCart();
        PaymentModeModel scalaPay = paymentModeService.getPaymentModeForCode(SCALA_PAY_CODE);
        cart.setPaymentMode(scalaPay);
        ScalaPayPaymentInfoModel paymentInfo = modelService.create(ScalaPayPaymentInfoModel.class);
        paymentInfo.setCode(customerModel.getUid() + "_" + UUID.randomUUID());
        paymentInfo.setUser(customerModel);
        paymentInfo.setOrderToken(orderToken);
        CartModel sessionCart = cartService.getSessionCart();
        sessionCart.setPaymentInfo(paymentInfo);
        modelService.saveAll(paymentInfo, sessionCart);
    }

    @Override
    public PaymentTransactionEntryModel capture(CommerceCheckoutParameter parameter, @Nullable String reference) {

        CartModel cartModel = parameter.getCart();

        PaymentTransactionEntryModel transactionEntryModel = null;

        final PaymentInfoModel paymentInfo = cartModel.getPaymentInfo();
        if (paymentInfo instanceof ScalaPayPaymentInfoModel) {
            ScalaPayPaymentInfoModel scalaPaymentInfo = (ScalaPayPaymentInfoModel) paymentInfo;
            boolean captured = false;
            OrderStatusResponse capture = scalapayService.capture(scalaPaymentInfo.getOrderToken(), reference);
            if (capture != null) {
                OrderStatus orderStatus = OrderStatus.valueOf(capture.getStatus());
                captured = orderStatus == OrderStatus.APPROVED || orderStatus == OrderStatus.AUTHORIZED;
            }
            transactionEntryModel = modelService.create(PaymentTransactionEntryModel.class);
            transactionEntryModel.setCode(scalaPaymentInfo.getOrderToken());
            transactionEntryModel.setTransactionStatus(captured ? TransactionStatus.ACCEPTED.name() :
                    TransactionStatus.REJECTED.name());

            if (TransactionStatus.ACCEPTED.name().equals(transactionEntryModel.getTransactionStatus())
                    || TransactionStatus.REVIEW.name().equals(transactionEntryModel.getTransactionStatus())) {
                final PaymentTransactionModel paymentTransaction = modelService.create(PaymentTransactionModel.class);
                cartModel.setPaymentStatus(PaymentStatus.PAID);
                paymentTransaction.setOrder(cartModel);
                paymentTransaction.setInfo(paymentInfo);
                transactionEntryModel.setPaymentTransaction(paymentTransaction);
                modelService.save(paymentTransaction);
            } else {
                cartModel.setPaymentStatus(PaymentStatus.NOTPAID);
            }
            modelService.save(cartModel);
        }
        return transactionEntryModel;
    }

    @Override
    public void updateMerchantReference(OrderModel order) {
        scalapayService.updateMerchantReference(order);
    }

    @Override
    public Integer getMinimumSupportedAmount() {
        return getScalaPayConfiguration().getMinAmount();
    }

    @Override
    public Integer getMaximumSupportedAmount() {
        return getScalaPayConfiguration().getMaxAmount();
    }

    @Override
    public Map<String, ScalaPayStylingData> getStylingData() {
        return scalaPayStylingConverter.convert(getScalaPayConfiguration());
    }

    private ScalaPayConfigurationModel getScalaPayConfiguration() {
        CartModel sessionCart = cartService.getSessionCart();
        return scalapayService.getScalaPayConfiguration(sessionCart);
    }

    @Required
    public void setScalapayService(ScalapayService scalapayService) {
        this.scalapayService = scalapayService;
    }

    @Required
    public void setCartService(CartService cartService) {
        this.cartService = cartService;
    }


    @Required
    public void setOrderTokenRequestConverter(Converter<CartModel, OrderTokenRequest> orderTokenRequestConverter) {
        this.orderTokenRequestConverter = orderTokenRequestConverter;
    }

    @Required
    public void setModelService(ModelService modelService) {
        this.modelService = modelService;
    }

    @Required
    public void setSessionService(SessionService sessionService) {
        this.sessionService = sessionService;
    }

    @Required
    public void setPaymentModeService(PaymentModeService paymentModeService) {
        this.paymentModeService = paymentModeService;
    }

    @Required
    public void setScalaPayStylingConverter(Converter<ScalaPayConfigurationModel, Map<String, ScalaPayStylingData>> scalaPayStylingConverter) {
        this.scalaPayStylingConverter = scalaPayStylingConverter;
    }
}
