/*
 * [y] hybris Platform
 *
 * Copyright (c) 2018 SAP SE or an SAP affiliate company. All rights reserved.
 *
 * This software is the confidential and proprietary information of SAP
 * ("Confidential Information"). You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms of the
 * license agreement you entered into with SAP.
 */
package com.scalapay.service.impl;

import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import com.scalapay.data.*;
import com.scalapay.model.ScalaPayConfigurationModel;
import com.scalapay.service.ScalapayService;
import de.hybris.platform.basecommerce.model.site.BaseSiteModel;
import de.hybris.platform.catalog.model.CatalogUnawareMediaModel;
import de.hybris.platform.commerceservices.impersonation.ImpersonationContext;
import de.hybris.platform.commerceservices.impersonation.ImpersonationService;
import de.hybris.platform.core.model.media.MediaModel;
import de.hybris.platform.core.model.order.AbstractOrderModel;
import de.hybris.platform.core.model.order.CartModel;
import de.hybris.platform.core.model.order.OrderModel;
import de.hybris.platform.core.model.order.payment.PaymentInfoModel;
import de.hybris.platform.core.model.order.payment.ScalaPayPaymentInfoModel;
import de.hybris.platform.order.CartService;
import de.hybris.platform.servicelayer.exceptions.SystemException;
import de.hybris.platform.servicelayer.media.MediaService;
import de.hybris.platform.servicelayer.model.ModelService;
import de.hybris.platform.servicelayer.search.FlexibleSearchQuery;
import de.hybris.platform.servicelayer.search.FlexibleSearchService;
import de.hybris.platform.site.BaseSiteService;
import de.hybris.platform.store.BaseStoreModel;
import de.hybris.platform.store.services.BaseStoreService;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.http.*;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestClientResponseException;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Nullable;
import javax.annotation.PostConstruct;
import java.io.InputStream;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.Collections;
import java.util.Optional;


public class DefaultScalapayService implements ScalapayService {

    private static final Logger LOG = LoggerFactory.getLogger(DefaultScalapayService.class);

    public static final String SUFFIX_ORDERS = "orders";
    public static final String SUFFIX_STATUS = "payments";
    public static final String SUFFIX_PAYMENTS = "payments";
    public static final String SUFFIX_CAPTURE = SUFFIX_PAYMENTS + "/capture";
    public static final String SUFFIX_DELAYED_CAPTURE = "/delay";
    public static final String POSTFIX_REFUND = "/refund";

    private MediaService mediaService;
    private ModelService modelService;
    private FlexibleSearchService flexibleSearchService;
    private BaseStoreService baseStoreService;
    private ImpersonationService impersonationService;
    private BaseSiteService baseSiteService;
    private CartService cartService;
    private RestTemplate scalaPayRestTemplate;

    private static final String FIND_ORDER_QUERY = "SELECT {" + OrderModel.PK + "} FROM {"
            + OrderModel._TYPECODE + "} WHERE {" + OrderModel.CODE + "}=?orderCode";

    private static final String FIND_ORDER_BY_TOKEN_QUERY = "SELECT {o." + AbstractOrderModel.PK + "} FROM " +
            "{" + ScalaPayPaymentInfoModel._TYPECODE + " as p}," +
            "{" + AbstractOrderModel._TYPECODE + " as o} WHERE " +
            "{ o." + AbstractOrderModel.PAYMENTINFO + "} = {p." + ScalaPayPaymentInfoModel.PK + "} AND " +
            "{" + ScalaPayPaymentInfoModel.ORDERTOKEN + "} = ?token";

    public LoadingCache<BaseStoreModel, HttpHeaders> headersCache;

    @PostConstruct
    public void setup() {
        headersCache = CacheBuilder.newBuilder()
                .maximumSize(10)
                .build(new CacheLoader<BaseStoreModel, HttpHeaders>() {
                    @Override
                    public HttpHeaders load(BaseStoreModel key) {
                        return getHeaders(key);
                    }
                });
    }

    public HttpHeaders getHeaders(BaseStoreModel store) { // todo caching
        HttpHeaders headers = new HttpHeaders();
        String token = this.getScalaPayConfiguration(store).getScalapayToken();
        headers.setAccept(Collections.singletonList(MediaType.APPLICATION_JSON));
        headers.set("Authorization", "Bearer " + token);
        return headers;
    }

    @Override
    public String getHybrisLogoUrl(final String logoCode) {
        final MediaModel media = mediaService.getMedia(logoCode);

        // Keep in mind that with Slf4j you don't need to check if debug is enabled, it is done under the hood.
        LOG.debug("Found media [code: {}]", media.getCode());

        return media.getURL();
    }

    @Override
    public void createLogo(final String logoCode) {
        final Optional<CatalogUnawareMediaModel> existingLogo = findExistingLogo(logoCode);

        final CatalogUnawareMediaModel media = existingLogo.orElseGet(() -> modelService.create(CatalogUnawareMediaModel.class));
        media.setCode(logoCode);
        media.setRealFileName("logo_dark.png");
        modelService.save(media);

        mediaService.setStreamForMedia(media, getImageStream());
    }

    @Override
    public OrderTokenResponse getOrderToken(OrderTokenRequest request, BaseStoreModel store) {
        ResponseEntity<OrderTokenResponse> tokenResponse;
        try {
            HttpEntity<OrderTokenRequest> entity = new HttpEntity<>(request, headersCache.getUnchecked(store));
            tokenResponse = scalaPayRestTemplate.postForEntity(getEndpointUrl(SUFFIX_ORDERS, store), entity, OrderTokenResponse.class);
        } catch (RestClientException e) {
            LOG.error("Error generating scala token {}", e.getMessage());
            if (e instanceof RestClientResponseException) {
                String responseBodyAsString = ((RestClientResponseException) e).getResponseBodyAsString();
                LOG.error(responseBodyAsString);
            }
            return null;
        }
        return tokenResponse.getBody();
    }

    @Override
    public OrderStatus getOrderStatus(String orderCode) {
        String token = getScalaTokenForOrderCode(orderCode);
        BaseStoreModel store = getOrderByCode(orderCode).getStore();
        try {
            ResponseEntity<OrderStatusResponse> statusResponse = scalaPayRestTemplate.exchange(
                    getEndpointUrl(SUFFIX_STATUS, store) + "/" + token, HttpMethod.GET,
                    new HttpEntity<>(headersCache.getUnchecked(store)), OrderStatusResponse.class);
            String status = statusResponse.getBody().getStatus();
            return OrderStatus.valueOf(status.toUpperCase());
        } catch (IllegalArgumentException | HttpClientErrorException e) {
            LOG.error("Error trying to get order status: " + e);
            return null;
        }

    }

    @Override
    public OrderStatusResponse capture(String token, @Nullable String reference) {
        AbstractOrderModel order = getOrderByScalaToken(token);
        BaseStoreModel store = order.getStore();
        HttpHeaders headers = headersCache.getUnchecked(store);
        CaptureRequest request = new CaptureRequest();
        String ref = StringUtils.defaultIfEmpty(reference, "to-be-updated-after-order-placed");
        request.setMerchantReference(ref);
        CartModel cart = cartService.getSessionCart();
        ScalaPayConfigurationModel scalaPayConfiguration = getScalaPayConfiguration(cart);
        boolean delayedCapture = scalaPayConfiguration.getDelayedCapture();
        try {
            ResponseEntity<OrderStatusResponse> response;
            if (!delayedCapture) {
                request.setToken(token);
                HttpEntity<CaptureRequest> entity = new HttpEntity<>(request, headers);
                response = scalaPayRestTemplate.postForEntity
                        (getEndpointUrl(SUFFIX_CAPTURE, store), entity, OrderStatusResponse.class);
            } else {
                HttpEntity<CaptureRequest> entity = new HttpEntity<>(request, headers);
                response = scalaPayRestTemplate.postForEntity
                        (getEndpointUrl(SUFFIX_DELAYED_CAPTURE, store), entity, OrderStatusResponse.class);
            }
            return response.getBody();

        } catch (IllegalArgumentException | HttpMessageNotReadableException e) {
            LOG.warn("Unsuccessful capture: " + e);
            return null;
        } catch (HttpClientErrorException e) {
            LOG.warn("Unsuccessful capture: " + e.getResponseBodyAsString());
            return null;
        }
    }

    @Override
    public void updateMerchantReference(OrderModel order) {
        UpdateRequest updateRequest = new UpdateRequest();
        updateRequest.setMerchantReference(order.getCode());
        String token = getScalaTokenForOrder(order);
        try {
            HttpEntity<UpdateRequest> entity = new HttpEntity<>(updateRequest, getHeaders(order.getStore()));
            ResponseEntity<UpdateResponse> orderStatusResponseResponseEntity = scalaPayRestTemplate.postForEntity
                    (getEndpointUrl(SUFFIX_ORDERS, order.getStore()) + "/" + token, entity, UpdateResponse.class);
            if (orderStatusResponseResponseEntity.getStatusCodeValue() != HttpStatus.OK.value()) {
                LOG.warn("Unsuccessful update: " + orderStatusResponseResponseEntity.getBody());
            }
            LOG.debug("Successful update: " + orderStatusResponseResponseEntity);
        } catch (IllegalArgumentException | HttpMessageNotReadableException e) {
            LOG.warn("Unsuccessful update: " + e);
        } catch (HttpClientErrorException e) {
            LOG.warn("Unsuccessful update: " + e.getResponseBodyAsString());
        }
    }

    private String getScalaTokenForOrder(OrderModel order) {
        PaymentInfoModel paymentInfo = order.getPaymentInfo();
        if (!(paymentInfo instanceof ScalaPayPaymentInfoModel)) {
            throw new IllegalArgumentException("Order " + order.getCode() + " is not a Scalapay order");
        }
        return ((ScalaPayPaymentInfoModel) paymentInfo).getOrderToken();
    }

    @Override
    public String refund(String orderCode, double amount, String currencyCode) {
        String token = getScalaTokenForOrderCode(orderCode);
        OrderModel order = getOrderByCode(orderCode);
        RefundRequest request = new RefundRequest();
        Amount refundAmount = new Amount();
        refundAmount.setAmount(new BigDecimal(amount).setScale(2, RoundingMode.HALF_UP).toString());
        refundAmount.setCurrency(currencyCode);
        request.setRefundAmount(refundAmount);
        try {
            HttpEntity<RefundRequest> entity = new HttpEntity<>(request, getHeaders(order.getStore()));
            String url = getEndpointUrl(SUFFIX_PAYMENTS, order.getStore()) + "/" + token + "/" + POSTFIX_REFUND;
            ResponseEntity<RefundResponse> response = scalaPayRestTemplate.postForEntity(
                    url, entity, RefundResponse.class);
            return response.getBody().getRefundId();
        } catch (RestClientException e) {
            return null;
        }
    }


    private OrderModel getOrderByCode(String orderCode) {
        final FlexibleSearchQuery fQuery = new FlexibleSearchQuery(FIND_ORDER_QUERY);
        fQuery.addQueryParameter("orderCode", orderCode);

        try {
            return flexibleSearchService.searchUnique(fQuery);
        } catch (final SystemException e) {
            throw new IllegalArgumentException("No order found for code " + orderCode);
        }
    }

    public AbstractOrderModel getOrderByScalaToken(String token) {
        final FlexibleSearchQuery fQuery = new FlexibleSearchQuery(FIND_ORDER_BY_TOKEN_QUERY);
        fQuery.addQueryParameter("token", token);
        try {
            return flexibleSearchService.searchUnique(fQuery);
        } catch (final SystemException e) {
            throw new IllegalArgumentException("No order found for token " + token);
        }
    }

    /**
     * Get the scala token for a Hybris order code.
     * @param orderCode The Hybris order code
     * @return The scalaToken if the order could be found, and if the order was paid with Scalapay
     */
    private String getScalaTokenForOrderCode(String orderCode) {
        OrderModel order = getOrderByCode(orderCode);
        PaymentInfoModel paymentInfo = order.getPaymentInfo();
        if (!(paymentInfo instanceof ScalaPayPaymentInfoModel)) {
            throw new IllegalArgumentException("Order " + orderCode + " is not a Scalapay order");
        }
        return ((ScalaPayPaymentInfoModel) paymentInfo).getOrderToken();
    }


    public ScalaPayConfigurationModel getScalaPayConfiguration(AbstractOrderModel orderModel) {
        BaseStoreModel store = orderModel.getStore();
        return getScalaPayConfiguration(store);
    }

    public ScalaPayConfigurationModel getScalaPayConfiguration(BaseStoreModel baseStore) {
        return baseStore.getScalapayConfiguration();
    }


    public ScalaPayConfigurationModel getScalaPayConfiguration() {
        CartModel sessionCart = cartService.getSessionCart();
        if (sessionCart == null) {
            final ImpersonationContext context = new ImpersonationContext();
            Optional<BaseSiteModel> first = baseSiteService.getAllBaseSites().stream().findFirst();
            context.setSite(first.get());

            return impersonationService.executeInContext(context, () -> {
                BaseStoreModel currentBaseStore = baseStoreService.getCurrentBaseStore();
                if (!currentBaseStore.getScalapayEnabled()) {
                    LOG.warn("Scalapay not enabled for store " + currentBaseStore.getName());
                }
                return currentBaseStore.getScalapayConfiguration();
            });
        }
        return getScalaPayConfiguration(sessionCart);
    }


    @Override
    public boolean isEnabled(BaseStoreModel baseStore) {
        return baseStore.getScalapayEnabled();
    }

    private String getEndpointUrl(String suffix, BaseStoreModel store) {
        ScalaPayConfigurationModel scalaPayConfiguration = getScalaPayConfiguration(store);
        return scalaPayConfiguration.getScalapayAPIUrl() + "/" + suffix;
    }

    private final static String FIND_LOGO_QUERY = "SELECT {" + CatalogUnawareMediaModel.PK + "} FROM {"
            + CatalogUnawareMediaModel._TYPECODE + "} WHERE {" + CatalogUnawareMediaModel.CODE + "}=?code";

    private Optional<CatalogUnawareMediaModel> findExistingLogo(final String logoCode) {
        final FlexibleSearchQuery fQuery = new FlexibleSearchQuery(FIND_LOGO_QUERY);
        fQuery.addQueryParameter("code", logoCode);

        try {
            return Optional.of(flexibleSearchService.searchUnique(fQuery));
        } catch (final SystemException e) {
            return Optional.empty();
        }
    }

    private InputStream getImageStream() {
        return DefaultScalapayService.class.getResourceAsStream("/scalapay/logo_dark.png");
    }

    @Required
    public void setMediaService(final MediaService mediaService) {
        this.mediaService = mediaService;
    }

    @Required
    public void setModelService(final ModelService modelService) {
        this.modelService = modelService;
    }

    @Required
    public void setFlexibleSearchService(final FlexibleSearchService flexibleSearchService) {
        this.flexibleSearchService = flexibleSearchService;
    }

    @Required
    public void setScalaPayRestTemplate(RestTemplate scalaPayRestTemplate) {
        this.scalaPayRestTemplate = scalaPayRestTemplate;
    }

    @Required
    public void setBaseStoreService(BaseStoreService baseStoreService) {
        this.baseStoreService = baseStoreService;
    }

    @Required
    public void setImpersonationService(ImpersonationService impersonationService) {
        this.impersonationService = impersonationService;
    }

    @Required
    public void setBaseSiteService(BaseSiteService baseSiteService) {
        this.baseSiteService = baseSiteService;
    }

    @Required
    public void setCartService(CartService cartService) {
        this.cartService = cartService;
    }
}

