/*
 * [y] hybris Platform
 *
 * Copyright (c) 2018 SAP SE or an SAP affiliate company. All rights reserved.
 *
 * This software is the confidential and proprietary information of SAP
 * ("Confidential Information"). You shall not disclose such Confidential
 * Information and shall use it only in accordance with the terms of the
 * license agreement you entered into with SAP.
 */
package com.scalapay.service.impl;

import com.scalapay.data.*;
import com.scalapay.service.ScalapayService;
import de.hybris.bootstrap.annotations.IntegrationTest;
import de.hybris.platform.core.model.media.MediaModel;
import de.hybris.platform.servicelayer.ServicelayerBaseTest;
import de.hybris.platform.servicelayer.search.FlexibleSearchQuery;
import de.hybris.platform.servicelayer.search.FlexibleSearchService;
import de.hybris.platform.store.services.BaseStoreService;
import org.junit.Before;
import org.junit.Test;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;


/**
 * This is an example of how the integration test should look like. {@link ServicelayerBaseTest} bootstraps platform so
 * you have an access to all Spring beans as well as database connection. It also ensures proper cleaning out of items
 * created during the test after it finishes. You can inject any Spring service using {@link Resource} annotation. Keep
 * in mind that by default it assumes that annotated field name matches the Spring Bean ID.
 */
@IntegrationTest
public class DefaultScalapayServiceIntegrationTest  extends ServicelayerBaseTest
{
	@Resource
	private ScalapayService scalapayService;
	@Resource
	private FlexibleSearchService flexibleSearchService;
	@Resource
	private BaseStoreService baseStoreService;

	@Before
	public void setUp() throws Exception
	{
		//scalapayService.createLogo(PLATFORM_LOGO_CODE);
	}

	@Test
	public void shouldReturnProperUrlForLogo() throws Exception
	{
		// given
		final String logoCode = "scalapayPlatformLogo";

		// when
		final String logoUrl = scalapayService.getHybrisLogoUrl(logoCode);

		// then
		assertTrue(true);
		//assertThat(logoUrl).isNotNull();
		//assertThat(logoUrl).isEqualTo(findLogoMedia(logoCode).getURL());
	}

	@Test
	public void shouldCreateAnOrderToken() {
		OrderTokenRequest request = createTokenRequest();
		OrderTokenResponse orderToken = scalapayService.getOrderToken(request, null);
		assertNotNull(orderToken);
		assertNotNull(orderToken.getToken());
	}

	@Test
	public void shouldCreatePayment() {
		OrderTokenRequest request = createTokenRequest();
		OrderTokenResponse orderToken = scalapayService.getOrderToken(request, null);
		String checkoutUrl = orderToken.getCheckoutUrl();
		System.out.println(checkoutUrl); // https://portal.scalapay.it/checkout?token=ORDER_TOKEN
		assertNotNull(checkoutUrl); // will need to follow this url to trigger auth
	}

	@Test
	public void shouldCapturePayment() {
		String token = "91K85LRK93";
		OrderStatusResponse capture = scalapayService.capture(token, "");
		assertNotNull(capture);
		OrderStatus orderStatus = OrderStatus.valueOf(capture.getStatus());
		assertEquals(orderStatus, OrderStatus.APPROVED);
	}

	@Test
	public void shouldShowOrderStatusAsPendingBeforeCapture() {
		OrderTokenRequest request = createTokenRequest();
		OrderTokenResponse orderToken = scalapayService.getOrderToken(request, null);
		OrderStatus orderStatus = scalapayService.getOrderStatus(orderToken.getToken());
		assertEquals(OrderStatus.PENDING, orderStatus);
	}

	@Test
	public void shouldFailGetOrderStatusAsPendingOnInvalidToken() {
		OrderStatus orderStatus = scalapayService.getOrderStatus("invalid");
		assertNull(orderStatus);
	}

	@Test
	public void testRefund_shouldNotWorkWithoutCapture() {
		OrderTokenRequest request = createTokenRequest();
		OrderTokenResponse orderToken = scalapayService.getOrderToken(request, null);
		String refundId = scalapayService.refund(orderToken.getToken(), Double.parseDouble(request.getTotalAmount().getAmount()),
				request.getTotalAmount().getCurrency());
		assertNull(refundId);
	}

	private OrderTokenRequest createTokenRequest() {
		OrderTokenRequest request = new OrderTokenRequest();
		request.setTotalAmount(createAmount(40.70));
		request.setConsumer(createConsumer());
		request.setBilling(createAddress());
		request.setShipping(createAddress());
		request.setItems(createItems());
		request.setDiscounts(createDiscount());
		request.setMerchant(createMerchant());
		request.setMerchantReference("merchantOrder-1234");
		request.setTaxAmount(createAmount(3.7));
		request.setShippingAmount(createAmount(10));
		request.setOrderExpiryMilliseconds(3600000);
		return request;
	}

	private Merchant createMerchant() {
		Merchant merchant = new Merchant();
		merchant.setRedirectConfirmUrl("https://staging.scalapay.it/success-url");
		merchant.setRedirectCancelUrl("https://staging.scalapay.it/failure-url");
		return merchant;
	}

	private List<Discount> createDiscount() {
		List<Discount> discounts = new ArrayList<>();
		Discount discount = new Discount();
		discount.setDisplayName("10% Off");
		Amount amount = createAmount(3);
		discount.setAmount(amount);
		discounts.add(discount);
		return discounts;
	}

	private List<Item> createItems() {
		List<Item> items = new ArrayList<>();
		items.add(createItem("T-Shirt", "12341234", 10));
		items.add(createItem("Jeans", "12341235", 20));
		return items;
	}

	private Item createItem(String name, String sku, double amount) {
		Item item = new Item();
		item.setName(name);
		item.setSku(sku);
		item.setQuantity(1);
		Amount price = createAmount(amount);
		item.setPrice(price);

		return item;


	}

	private Address createAddress() {
		Address address = new Address();
		address.setName("Joe Consumer");
		address.setLine1("Via della Rosa, 23");
		address.setSuburb("Montelupo Fiorentino");
		address.setPostcode("50056");
		address.setCountryCode("IT");
		address.setPhoneNumber("0400000000");
		return address;
	}

	private Customer createConsumer() {
		Customer customer = new Customer();
		customer.setPhoneNumber("0400000001");
		customer.setGivenNames("Joe");
		customer.setSurname("Consumer");
		customer.setEmail("test@scalapay.it");
		return customer;
	}

	private Amount createAmount(double amount) {
		Amount totalAmount = new Amount();
		totalAmount.setAmount(new BigDecimal(amount).setScale(2, RoundingMode.HALF_UP).toString());
		totalAmount.setCurrency("EUR");
		return totalAmount;
	}


	private MediaModel findLogoMedia(final String logoCode)
	{
		final FlexibleSearchQuery fQuery = new FlexibleSearchQuery("SELECT {PK} FROM {Media} WHERE {code}=?code");
		fQuery.addQueryParameter("code", logoCode);

		return flexibleSearchService.searchUnique(fQuery);
	}

}
